<?php

namespace Drupal\lazy_image_style\Plugin\Filter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Annotation\Translation;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Drupal\lazy_image_style\LazyImageStyleHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'Track supported lazy images' filter.
 *
 * @Filter(
 *   id = "lazy_image_style",
 *   title = @Translation("Track supported lazy images"),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_HTML_RESTRICTOR,
 *   description = @Translation("Attaching lazy load library if data-src attribute found."),
 *   weight = 10
 * )
 */
class LazyImageStyle extends FilterBase implements ContainerFactoryPluginInterface {

  /**
   * The lazy image style helper service.
   *
   * @var \Drupal\lazy_image_style\LazyImageStyleHelper
   */
  protected $helper;

  /**
   * LazyImageStyle constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param $plugin_id
   *   The plugin_id for the plugin instance.
   * @param $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\lazy_image_style\LazyImageStyleHelper $helper
   *   The lazy image style helper service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LazyImageStyleHelper $helper) {
    $this->helper = $helper;
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('lazy_image_style.helper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {
    $result = new FilterProcessResult($text);

    if (stristr($text, 'data-use_lazy') === FALSE) {
      return $result;
    }
    $dom = HTML::load($text);

    /** @var \DOMElement $item */
    foreach ($dom->getElementsByTagName('img') as $item) {
      if ($item->hasAttribute('data-use_lazy')) {
        $item->setAttribute('data-src', $item->getAttribute('src'));
        $item->setAttribute('src', $this->helper::TMP_BASE64_IMAGE);
        $item->removeAttribute('data-use_lazy');
      }
    }
    $result
      ->setProcessedText(Html::serialize($dom))
      ->addAttachments($this->helper->addAttachments());
    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function tips($long = FALSE) {
    return $this->t('Some filter tips here.');
  }
}
